from django.test import TestCase

from squads.forms import SquadsGroupForm


class SquadsGroupFormTest(TestCase):
    def test_clean_description_too_long(self):
        # given
        form_data = {
            "name": "Test Group",
            "description": "a" * 1001,
            "require_approval": False,
        }
        # when
        form = SquadsGroupForm(data=form_data)
        # then
        self.assertFalse(form.is_valid())
        self.assertIn("description", form.errors)
        self.assertEqual(
            form.errors["description"][0],
            "The Description is too long. Please keep it under 1000 characters.",
        )

    def test_clean_description_valid(self):
        # given
        form_data = {
            "name": "Test Group",
            "description": "<p>This is a valid description.</p>",
            "require_approval": False,
        }
        # when
        form = SquadsGroupForm(data=form_data)
        # then
        self.assertTrue(form.is_valid())

    def test_clean_description_invalid_tags(self):
        # given
        form_data = {
            "name": "Test Group",
            "description": '<script>alert("Hack!");</script><p>This is a valid part of the description.</p>',
            "require_approval": False,
        }
        # when
        form = SquadsGroupForm(data=form_data)
        # then
        self.assertTrue(form.is_valid())
        cleaned_description = form.cleaned_data["description"]
        self.assertNotIn("<script>", cleaned_description)
        self.assertIn(
            "<p>This is a valid part of the description.</p>", cleaned_description
        )



"""Manage views."""

from django.contrib import messages
from django.contrib.auth.decorators import login_required, permission_required
from django.shortcuts import get_object_or_404, redirect

from squads.forms import CommentForm
from squads.hooks import get_extension_logger
from squads.models.groups import Groups, Pending
from squads.models.memberships import Memberships

logger = get_extension_logger(__name__)


@login_required
@permission_required("squads.basic_access")
def apply_group(request, group_id):
    group = get_object_or_404(Groups, id=group_id)
    comment_form = CommentForm(request.POST or None)

    user_has_required_skills = True

    if user_has_required_skills:
        user = request.user
        comment = None
        if comment_form.is_valid():
            comment = comment_form.cleaned_data.get("comment")
        Pending.objects.create(group=group, user=user, application=comment)
        messages.success(request, "Your application has been submitted.")
    else:
        messages.error(
            request, "You do not meet the skill requirements for this Squad."
        )

    return redirect("squads:view_group", group_id=group_id)


@login_required
@permission_required("squads.basic_access")
def leave_group(request, group_id):
    group = get_object_or_404(Groups, id=group_id)
    membership = Memberships.objects.filter(user=request.user, group=group).first()

    if membership:
        membership.delete()
        messages.success(request, f"You Left {membership.group.name}.")
    else:
        messages.error(request, "You are not in that Squad.")

    return redirect("squads:view_group", group_id=group_id)


@login_required
@permission_required("squads.basic_access")
def cancel_group(request, group_id):
    group = get_object_or_404(Groups, id=group_id)
    pending = Pending.objects.filter(user=request.user, group=group).first()

    if pending:
        pending.delete()
        messages.success(
            request, f"You canceled your application to {pending.group.name}."
        )
    else:
        messages.error(request, "You are not applied to that Squad.")

    return redirect("squads:view_group", group_id=group_id)



"""
App Settings
"""

# Django
from app_utils.app_settings import clean_setting

TESTING_MODE = False
# Set Test Mode True or False

# Set Naming on Auth Hook
SQUADS_APP_NAME = clean_setting("SQUADS_APP_NAME", "Squads")

# Caching Key for Caching System
STORAGE_BASE_KEY = "squads_storage_"

# zKillboard - https://zkillboard.com/
EVE_BASE_URL = "https://esi.evetech.net/"
EVE_API_URL = "https://esi.evetech.net/latest/"
EVE_BASE_URL_REGEX = r"^http[s]?:\/\/esi.evetech\.net\/"

# If True you need to set up the Logger
SQUADS_LOGGER_USE = clean_setting("SQUADS_LOGGER_USE", False)

# Empty Image
SQUADS_EMPTY_IMAGE = clean_setting(
    "SQUADS_EMPTY_IMAGE", "static/squads/groups_images/empty.png"
)



from http import HTTPStatus

from django.test import TestCase
from django.test.client import RequestFactory
from django.urls import reverse

from app_utils.testing import create_user_from_evecharacter

from squads.tests.testdata.load_allianceauth import load_allianceauth
from squads.views.main import squads_index


class TestViews(TestCase):
    @classmethod
    def setUpClass(cls):
        super().setUpClass()
        load_allianceauth()
        cls.factory = RequestFactory()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "squads.basic_access",
            ],
        )

    def test_view(self):
        request = self.factory.get(reverse("squads:index"))
        request.user = self.user
        response = squads_index(request)
        self.assertEqual(response.status_code, HTTPStatus.OK)



from unittest import TestCase
from unittest.mock import MagicMock

from django.core.exceptions import ValidationError

from app_utils.testing import create_user_from_evecharacter

from squads.forms import SquadsGroupForm
from squads.models import Groups
from squads.tests.testdata.load_allianceauth import load_allianceauth


class TestCleanImage(TestCase):
    @classmethod
    def setUpClass(cls) -> None:
        super().setUpClass()
        load_allianceauth()
        cls.user, cls.character_ownership = create_user_from_evecharacter(
            1001,
            permissions=[
                "squads.basic_access",
            ],
        )
        cls.form = SquadsGroupForm()
        cls.group = Groups(
            owner=cls.user,
            name="Test Group",
            description="Test Description",
            require_approval=True,
        )
        cls.group.save()
        cls.form = SquadsGroupForm(instance=cls.group)

    def test_init_with_instance_pk(self):
        self.assertIn("is_active", self.form.fields)
        self.assertFalse(self.form.fields["is_active"].required)
        self.assertEqual(self.form.fields["is_active"].label, "Is Active")
        self.assertEqual(self.form.fields["is_active"].initial, self.group.is_active)
        self.assertEqual(
            self.form.fields["is_active"].widget.attrs, {"class": "form-check-input"}
        )

    def test_clean_image_valid_size(self):
        # Mock image with size less than 2 MB
        image_mock = MagicMock()
        image_mock.size = 1 * 1024 * 1024  # 1 MB

        self.form.cleaned_data = {"image": image_mock}

        try:
            result = self.form.clean_image()
            self.assertEqual(result, image_mock)
        except ValidationError:
            self.fail("clean_image() raised ValidationError unexpectedly!")

    def test_clean_image_too_large(self):
        # Mock image with size greater than 2 MB
        image_mock = MagicMock()
        image_mock.size = 3 * 1024 * 1024  # 3 MB

        self.form.cleaned_data = {"image": image_mock}

        with self.assertRaises(ValidationError):
            self.form.clean_image()

    def test_clean_image_no_image(self):
        self.form.cleaned_data = {"image": None}

        result = self.form.clean_image()
        self.assertIsNone(result)
